@@ -65,15 +65,20 @@ class ApplicationCredentialRoleModel(sql.ModelBase, sql.ModelDictMixin):
 
 class AccessRuleModel(sql.ModelBase, sql.ModelDictMixin):
     __tablename__ = 'access_rule'
-    attributes = ['service', 'path', 'method']
+    attributes = ['external_id', 'user_id', 'service', 'path', 'method']
     id = sql.Column(sql.Integer, primary_key=True, nullable=False)
+    external_id = sql.Column(sql.String(64), index=True, unique=True)
+    user_id = sql.Column(sql.String(64), index=True)
     service = sql.Column(sql.String(64))
     path = sql.Column(sql.String(128))
     method = sql.Column(sql.String(16))
+    __table_args__ = (
+        sql.UniqueConstraint('user_id', 'service', 'path', 'method',
+                             name='duplicate_access_rule_for_user_constraint'),
+    )
     application_credential = sqlalchemy.orm.relationship(
         'ApplicationCredentialAccessRuleModel',
-        backref=sqlalchemy.orm.backref('access_rule'),
-        cascade='all, delete-orphan')
+        backref=sqlalchemy.orm.backref('access_rule'))
 
 
 class ApplicationCredentialAccessRuleModel(sql.ModelBase, sql.ModelDictMixin):
@@ -135,17 +140,25 @@ class ApplicationCredential(base.ApplicationCredentialDriverBase):
                 session.add(app_cred_role)
             if access_rules:
                 for access_rule in access_rules:
-                    access_rule_ref = AccessRuleModel.from_dict(access_rule)
-                    session.add(access_rule_ref)
+                    access_rule_ref = session.query(AccessRuleModel).filter_by(
+                        external_id=access_rule['id']).first()
+                    if not access_rule_ref:
+                        access_rule_ref = session.query(AccessRuleModel).filter_by(
+                            user_id=app_cred['user_id'],
+                            service=access_rule['service'],
+                            path=access_rule['path'],
+                            method=access_rule['method']).first()
+                    if not access_rule_ref:
+                        access_rule_ref = AccessRuleModel.from_dict({
+                            k.replace('id', 'external_id'): v
+                            for k, v in access_rule.items()})
+                        access_rule_ref['user_id'] = app_cred['user_id']
+                        session.add(access_rule_ref)
                     app_cred_access_rule = ApplicationCredentialAccessRuleModel()
                     app_cred_access_rule.application_credential = ref
                     app_cred_access_rule.access_rule = access_rule_ref
                     session.add(app_cred_access_rule)
-            application_credential_dict = ref.to_dict()
-            application_credential_dict.pop('internal_id')
-            application_credential_dict['roles'] = roles
-            if access_rules is not None:
-                application_credential_dict['access_rules'] = access_rules
+            application_credential_dict = self._to_dict(ref)
             return application_credential_dict
 
     def _to_dict(self, ref):
@@ -154,13 +167,21 @@ class ApplicationCredential(base.ApplicationCredentialDriverBase):
         app_cred['roles'] = roles
         if ref.access_rules:
             access_rules = [
-                {k: v for k, v in c.access_rule.to_dict().items() if k != 'id'}
+                self._access_rule_to_dict(c.access_rule)
                 for c in ref.access_rules
             ]
             app_cred['access_rules'] = access_rules
         app_cred.pop('internal_id')
         return app_cred
 
+    def _access_rule_to_dict(self, ref):
+        access_rule = ref.to_dict()
+        return {
+            k.replace('external_id', 'id'): v
+            for k, v in access_rule.items()
+            if k != 'user_id' and k != 'id'
+        }
+
     def get_application_credential(self, application_credential_id):
         with sql.session_for_read() as session:
             query = session.query(ApplicationCredentialModel).filter_by(
@@ -204,3 +225,37 @@ class ApplicationCredential(base.ApplicationCredentialDriverBase):
             query = query.filter_by(user_id=user_id)
             query = query.filter_by(project_id=project_id)
             query.delete()
+
+    def get_access_rule(self, access_rule_id):
+        with sql.session_for_read() as session:
+            query = session.query(AccessRuleModel).filter_by(
+                external_id=access_rule_id)
+            ref = query.first()
+            if not ref:
+                raise exception.AccessRuleNotFound(
+                    access_rule_id=access_rule_id)
+            access_rule = self._access_rule_to_dict(ref)
+            return access_rule
+
+    def list_access_rules_for_user(self, user_id, hints):
+        with sql.session_for_read() as session:
+            query = session.query(AccessRuleModel).filter_by(user_id=user_id)
+            refs = sql.filter_limit_query(AccessRuleModel, query, hints)
+            return [self._access_rule_to_dict(ref) for ref in refs]
+
+    def delete_access_rule(self, access_rule_id):
+        try:
+            with sql.session_for_write() as session:
+                query = session.query(AccessRuleModel)
+                ref = query.filter_by(external_id=access_rule_id).first()
+                if not ref:
+                    raise exception.AccessRuleNotFound(
+                        access_rule_id=access_rule_id)
+                session.delete(ref)
+        except AssertionError:
+            raise exception.ForbiddenNotSecurity("May not delete access rule in use")
+
+    def delete_access_rules_for_user(self, user_id):
+        with sql.session_for_write() as session:
+            query = session.query(AccessRuleModel).filter_by(user_id=user_id)
+            query.delete()
