@@ -12,17 +12,16 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import flask
 from oslo_log import log
 import six
 
 from keystone.auth.plugins import base
 from keystone.auth.plugins import mapped
 from keystone.common import provider_api
-from keystone.common import wsgi
 import keystone.conf
 from keystone import exception
 from keystone.i18n import _
-from keystone.models import token_model
 
 
 LOG = log.getLogger(__name__)
@@ -35,35 +34,32 @@ class Token(base.AuthMethodHandler):
 
     def _get_token_ref(self, auth_payload):
         token_id = auth_payload['id']
-        response = PROVIDERS.token_provider_api.validate_token(token_id)
-        return token_model.KeystoneToken(token_id=token_id,
-                                         token_data=response)
+        return PROVIDERS.token_provider_api.validate_token(token_id)
 
-    def authenticate(self, request, auth_payload):
+    def authenticate(self, auth_payload):
         if 'id' not in auth_payload:
             raise exception.ValidationError(attribute='id',
                                             target='token')
-        token_ref = self._get_token_ref(auth_payload)
-        if token_ref.is_federated_user and PROVIDERS.federation_api:
+        token = self._get_token_ref(auth_payload)
+        if token.is_federated and PROVIDERS.federation_api:
             response_data = mapped.handle_scoped_token(
-                request, token_ref, PROVIDERS.federation_api,
+                token, PROVIDERS.federation_api,
                 PROVIDERS.identity_api
             )
         else:
-            response_data = token_authenticate(request,
-                                               token_ref)
+            response_data = token_authenticate(token)
 
         # NOTE(notmorgan): The Token auth method is *very* special and sets the
         # previous values to the method_names. This is because it can be used
         # for re-scoping and we want to maintain the values. Most
         # AuthMethodHandlers do no such thing and this is not required.
-        response_data.setdefault('method_names', []).extend(token_ref.methods)
+        response_data.setdefault('method_names', []).extend(token.methods)
 
         return base.AuthHandlerResponse(status=True, response_body=None,
                                         response_data=response_data)
 
 
-def token_authenticate(request, token_ref):
+def token_authenticate(token):
     response_data = {}
     try:
 
@@ -72,24 +68,25 @@ def token_authenticate(request, token_ref):
         # state in Keystone. To do so is to invite elevation of
         # privilege attacks
 
-        project_scoped = 'project' in request.json_body['auth'].get(
+        json_body = flask.request.get_json(silent=True, force=True) or {}
+        project_scoped = 'project' in json_body['auth'].get(
             'scope', {}
         )
-        domain_scoped = 'domain' in request.json_body['auth'].get(
+        domain_scoped = 'domain' in json_body['auth'].get(
             'scope', {}
         )
 
-        if token_ref.oauth_scoped:
+        if token.oauth_scoped:
             raise exception.ForbiddenAction(
                 action=_(
                     'Using OAuth-scoped token to create another token. '
                     'Create a new OAuth-scoped token instead'))
-        elif token_ref.trust_scoped:
+        elif token.trust_scoped:
             raise exception.ForbiddenAction(
                 action=_(
                     'Using trust-scoped token to create another token. '
                     'Create a new trust-scoped token instead'))
-        elif token_ref.system_scoped and (project_scoped or domain_scoped):
+        elif token.system_scoped and (project_scoped or domain_scoped):
             raise exception.ForbiddenAction(
                 action=_(
                     'Using a system-scoped token to create a project-scoped '
@@ -99,12 +96,10 @@ def token_authenticate(request, token_ref):
 
         if not CONF.token.allow_rescope_scoped_token:
             # Do not allow conversion from scoped tokens.
-            if token_ref.project_scoped or token_ref.domain_scoped:
+            if token.project_scoped or token.domain_scoped:
                 raise exception.ForbiddenAction(
                     action=_('rescope a scoped token'))
 
-        wsgi.validate_token_bind(request.context_dict, token_ref)
-
         # New tokens maintain the audit_id of the original token in the
         # chain (if possible) as the second element in the audit data
         # structure. Look for the last element in the audit data structure
@@ -112,7 +107,7 @@ def token_authenticate(request, token_ref):
         # a token that has not been rescoped) or the audit_chain id (in
         # the case of a token that has been rescoped).
         try:
-            token_audit_id = token_ref.get('audit_ids', [])[-1]
+            token_audit_id = token.parent_audit_id or token.audit_id
         except IndexError:
             # NOTE(morganfainberg): In the case this is a token that was
             # issued prior to audit id existing, the chain is not tracked.
@@ -122,13 +117,9 @@ def token_authenticate(request, token_ref):
         # token expiration time is maintained in the new token. Not doing this
         # would make it possible for a user to continuously bump token
         # expiration through token rescoping without proving their identity.
-        response_data.setdefault('expires_at', token_ref.expires)
+        response_data.setdefault('expires_at', token.expires_at)
         response_data['audit_id'] = token_audit_id
-        response_data.setdefault('user_id', token_ref.user_id)
-        # TODO(morganfainberg: determine if token 'extras' can be removed
-        # from the response_data
-        response_data.setdefault('extras', {}).update(
-            token_ref.get('extras', {}))
+        response_data.setdefault('user_id', token.user_id)
 
         return response_data
 
