@@ -56,6 +56,8 @@ class Bootstrapper(object):
         self.default_domain_id = None
         self.admin_user_id = None
 
+        self.immutable_roles = False
+
     def bootstrap(self):
         # NOTE(morganfainberg): Ensure the default domain is in-fact created
         self._bootstrap_default_domain()
@@ -114,8 +116,18 @@ class Bootstrapper(object):
         try:
             role_id = uuid.uuid4().hex
             role = {'name': role_name, 'id': role_id}
+            if self.immutable_roles:
+                role['options'] = {'immutable': True}
             role = PROVIDERS.role_api.create_role(role_id, role)
             LOG.info('Created role %s', role_name)
+            if not self.immutable_roles:
+                LOG.warning("Role %(role)s was created as a mutable role. It "
+                            "is recommended to make this role immutable, "
+                            "which will become the default behavior of the "
+                            "bootstrap command in the future.You can opt into "
+                            "this behavior by using the --immutable-role "
+                            "flag, or update role %(role)s with the "
+                            "'immutable' resource option.", {'role': role_name})
             return role
         except exception.Conflict:
             LOG.info('Role %s exists, skipping creation.', role_name)
@@ -226,14 +238,14 @@ class Bootstrapper(object):
                 project_id=self.project_id,
                 role_id=self.admin_role_id
             )
-            LOG.info('Granted %(role)s on %(project)s to user'
-                     ' %(username)s.',
+            LOG.info('Granted role %(role)s on project %(project)s to '
+                     'user %(username)s.',
                      {'role': self.admin_role_name,
                       'project': self.project_name,
                       'username': self.admin_username})
         except exception.Conflict:
-            LOG.info('User %(username)s already has %(role)s on '
-                     '%(project)s.',
+            LOG.info('User %(username)s already has role %(role)s on '
+                     'project %(project)s.',
                      {'username': self.admin_username,
                       'role': self.admin_role_name,
                       'project': self.project_name})
@@ -249,12 +261,12 @@ class Bootstrapper(object):
             PROVIDERS.assignment_api.create_system_grant_for_user(
                 self.admin_user_id, self.admin_role_id
             )
-            LOG.info('Granted %(role)s on the system to user'
+            LOG.info('Granted role %(role)s on the system to user'
                      ' %(username)s.',
                      {'role': self.admin_role_name,
                       'username': self.admin_username})
         except exception.Conflict:
-            LOG.info('User %(username)s already has %(role)s on '
+            LOG.info('User %(username)s already has role %(role)s on '
                      'the system.',
                      {'username': self.admin_username,
                       'role': self.admin_role_name})
@@ -325,9 +337,10 @@ class Bootstrapper(object):
                     LOG.info('Created %(interface)s endpoint %(url)s',
                              {'interface': interface, 'url': url})
                 else:
-                    # NOTE(jamielennox): electing not to update existing
-                    # endpoints here. There may be call to do so in future.
-                    LOG.info('Skipping %s endpoint as already created',
-                             interface)
+                    endpoint_ref['url'] = url
+                    PROVIDERS.catalog_api.update_endpoint(
+                        endpoint_id=endpoint_ref['id'],
+                        endpoint_ref=endpoint_ref)
+                    LOG.info('%s endpoint updated', interface)
 
                 self.endpoints[interface] = endpoint_ref['id']
