@@ -15,6 +15,7 @@
 from __future__ import absolute_import
 from __future__ import print_function
 
+import argparse
 import datetime
 import os
 import sys
@@ -111,6 +112,14 @@ class BootStrap(BaseApp):
                             help=('The initial region_id endpoints will be '
                                   'placed in during the keystone bootstrap '
                                   'process.'))
+        parser.add_argument('--immutable-roles',
+                            default=False,
+                            action='store_true',
+                            help=('Whether default roles (admin, member, and '
+                                  'reader) should be immutable. Immutable '
+                                  'default roles is currently an opt-in '
+                                  'behavior, but will become the default in '
+                                  'future releases.'))
         return parser
 
     def do_bootstrap(self):
@@ -152,9 +161,9 @@ class BootStrap(BaseApp):
         self.endpoints = None
 
         if self.password is None:
-            print(_('Either --bootstrap-password argument or '
+            print(_('ERROR: Either --bootstrap-password argument or '
                     'OS_BOOTSTRAP_PASSWORD must be set.'))
-            raise ValueError
+            sys.exit(1)
 
         self.bootstrapper.admin_password = self.password
         self.bootstrapper.admin_username = self.username
@@ -166,6 +175,7 @@ class BootStrap(BaseApp):
         self.bootstrapper.public_url = self.public_url
         self.bootstrapper.internal_url = self.internal_url
         self.bootstrapper.region_id = self.region_id
+        self.bootstrapper.immutable_roles = CONF.command.immutable_roles
 
         self.bootstrapper.bootstrap()
         self.reader_role_id = self.bootstrapper.reader_role_id
@@ -1207,26 +1217,28 @@ class MappingEngineTester(BaseApp):
         parser = super(MappingEngineTester,
                        cls).add_argument_parser(subparsers)
 
+        parser.formatter_class = argparse.RawTextHelpFormatter
         parser.add_argument('--rules', default=None, required=True,
                             help=("Path to the file with "
                                   "rules to be executed. "
-                                  "Content must be a proper JSON structure, "
-                                  "with a top-level key 'rules' and "
+                                  "Content must be\na proper JSON structure, "
+                                  "with a top-level key 'rules' and\n"
                                   "corresponding value being a list."))
         parser.add_argument('--input', default=None, required=True,
                             help=("Path to the file with input attributes. "
-                                  "The content consists of ':' separated "
-                                  "parameter names and their values. "
-                                  "There is only one key-value pair per line. "
-                                  "A ';' in the value is a separator and then "
-                                  "a value is treated as a list. Example:\n "
-                                  "EMAIL: me@example.com\n"
-                                  "LOGIN: me\n"
-                                  "GROUPS: group1;group2;group3"))
+                                  "The content\nconsists of ':' separated "
+                                  "parameter names and their values.\nThere "
+                                  "is only one key-value pair per line. "
+                                  "A ';' in the\nvalue is a separator and "
+                                  "then a value is treated as a list.\n"
+                                  "Example:\n"
+                                  "\tEMAIL: me@example.com\n"
+                                  "\tLOGIN: me\n"
+                                  "\tGROUPS: group1;group2;group3"))
         parser.add_argument('--prefix', default=None,
                             help=("A prefix used for each environment "
-                                  "variable in the assertion. For example, "
-                                  "all environment variables may have the "
+                                  "variable in the\nassertion. For example, "
+                                  "all environment variables may have\nthe "
                                   "prefix ASDF_."))
         parser.add_argument('--engine-debug',
                             default=False, action="store_true",
