@@ -429,6 +429,10 @@ class DomainSpecificRoleNotWithinIdPDomain(Forbidden):
                        "the identity provider: %(identity_provider)s.")
 
 
+class DomainIdInvalid(ValidationError):
+    message_format = _("Domain ID does not conform to required UUID format.")
+
+
 class RoleAssignmentNotFound(NotFound):
     message_format = _("Could not find role assignment with role: "
                        "%(role_id)s, user or group: %(actor_id)s, "
@@ -545,9 +549,8 @@ class ApplicationCredentialNotFound(NotFound):
                        "%(application_credential_id)s.")
 
 
-class AccessRulesConfigNotFound(NotFound):
-    message_format = _(
-        "Could not find access rules config for service %(service)s.")
+class AccessRuleNotFound(NotFound):
+    message_format = _("Could not find Access Rule: %(access_rule_id)s.")
 
 
 class Conflict(Error):
@@ -585,12 +588,6 @@ class TrustConsumeMaximumAttempt(UnexpectedError):
                              "acquire lock.")
 
 
-class CertificateFilesUnavailable(UnexpectedError):
-    debug_message_format = _("Expected signing certificates are not available "
-                             "on the server. Please check Keystone "
-                             "configuration.")
-
-
 class MalformedEndpoint(UnexpectedError):
     debug_message_format = _("Malformed endpoint URL (%(endpoint)s),"
                              " see ERROR log for details.")
@@ -712,6 +709,13 @@ class CacheDeserializationError(Exception):
         )
 
 
-class AccessRulesConfigFileError(UnexpectedError):
-    debug_message_format = _(
-        'Could not parse access rules config file: %(error)s')
+class ResourceUpdateForbidden(ForbiddenNotSecurity):
+    message_format = _('Unable to update immutable %(type)s resource: '
+                       '`%(resource_id)s. Set resource option "immutable" '
+                       'to false first.')
+
+
+class ResourceDeleteForbidden(ForbiddenNotSecurity):
+    message_format = _('Unable to delete immutable %(type)s resource: '
+                       '`%(resource_id)s. Set resource option "immutable" '
+                       'to false first.')
