@@ -78,7 +78,20 @@ service_provider_update = {
     'additionalProperties': False
 }
 
-_identity_provider_properties = {
+_identity_provider_properties_create = {
+    'enabled': parameter_types.boolean,
+    'description': validation.nullable(parameter_types.description),
+    'domain_id': validation.nullable(parameter_types.id_string),
+    'remote_ids': {
+        'type': ['array', 'null'],
+        'items': {
+            'type': 'string'
+        },
+        'uniqueItems': True
+    }
+}
+
+_identity_provider_properties_update = {
     'enabled': parameter_types.boolean,
     'description': validation.nullable(parameter_types.description),
     'remote_ids': {
@@ -92,24 +105,38 @@ _identity_provider_properties = {
 
 identity_provider_create = {
     'type': 'object',
-    'properties': _identity_provider_properties,
+    'properties': _identity_provider_properties_create,
     'additionalProperties': False
 }
 
 identity_provider_update = {
     'type': 'object',
-    'properties': _identity_provider_properties,
+    'properties': _identity_provider_properties_update,
     # Make sure at least one property is being updated
     'minProperties': 1,
     'additionalProperties': False
 }
 
-federation_protocol_schema = {
+_remote_id_attribute_properties = {
+    'type': 'string',
+    'maxLength': 64,
+}
+
+_protocol_properties = {
+    'mapping_id': parameter_types.mapping_id_string,
+    'remote_id_attribute': _remote_id_attribute_properties
+}
+
+protocol_create = {
     'type': 'object',
-    'properties': {
-        'mapping_id': parameter_types.mapping_id_string
-    },
-    # `mapping_id` is the property that cannot be ignored
+    'properties': _protocol_properties,
+    'required': ['mapping_id'],
+    'additionalProperties': False
+}
+
+protocol_update = {
+    'type': 'object',
+    'properties': _protocol_properties,
     'minProperties': 1,
     'additionalProperties': False
 }
