@@ -28,7 +28,6 @@ from pycadf import reason
 
 from keystone import assignment  # TODO(lbragstad): Decouple this dependency
 from keystone.common import cache
-from keystone.common import clean
 from keystone.common import driver_hints
 from keystone.common import manager
 from keystone.common import provider_api
@@ -111,9 +110,8 @@ class DomainConfigs(provider_api.ProviderAPIMixin, dict):
         try:
             domain_ref = resource_api.get_domain_by_name(domain_name)
         except exception.DomainNotFound:
-            LOG.warning(
-                ('Invalid domain name (%s) found in config file name'),
-                domain_name)
+            LOG.warning('Invalid domain name (%s) found in config file name',
+                        domain_name)
             return
 
         # Create a new entry in the domain config dict, which contains
@@ -502,14 +500,6 @@ class Manager(manager.Manager):
 
         driver = self._select_identity_driver(domain_id)
 
-        if not driver.is_sql:
-            # The LDAP driver does not support deleting users or groups.
-            # Moreover, we shouldn't destroy users and groups in an unknown
-            # driver. The only time when we should delete users and groups is
-            # when the backend is SQL because the foreign key in the SQL table
-            # forces us to.
-            return
-
         user_refs = self.list_users(domain_scope=domain_id)
         group_refs = self.list_groups(domain_scope=domain_id)
 
@@ -526,7 +516,10 @@ class Manager(manager.Manager):
         # And finally, delete the users themselves
         for user in user_refs:
             try:
-                self.delete_user(user['id'])
+                if not driver.is_sql:
+                    PROVIDERS.shadow_users_api.delete_user(user['id'])
+                else:
+                    self.delete_user(user['id'])
             except exception.UserNotFound:
                 LOG.debug(('User %(userid)s not found when deleting domain '
                            'contents for %(domainid)s, continuing with '
@@ -749,13 +742,13 @@ class Manager(manager.Manager):
         if (not driver.is_domain_aware() and driver == self.driver and
             domain_id != CONF.identity.default_domain_id and
                 domain_id is not None):
-                    LOG.warning('Found multiple domains being mapped to a '
-                                'driver that does not support that (e.g. '
-                                'LDAP) - Domain ID: %(domain)s, '
-                                'Default Driver: %(driver)s',
-                                {'domain': domain_id,
-                                 'driver': (driver == self.driver)})
-                    raise exception.DomainNotFound(domain_id=domain_id)
+            LOG.warning('Found multiple domains being mapped to a '
+                        'driver that does not support that (e.g. '
+                        'LDAP) - Domain ID: %(domain)s, '
+                        'Default Driver: %(driver)s',
+                        {'domain': domain_id,
+                         'driver': (driver == self.driver)})
+            raise exception.DomainNotFound(domain_id=domain_id)
         return driver
 
     def _get_domain_driver_and_entity_id(self, public_id):
@@ -900,11 +893,10 @@ class Manager(manager.Manager):
     # - select the right driver for this domain
     # - clear/set domain_ids for drivers that do not support domains
     # - create any ID mapping that might be required
-
     @notifications.emit_event('authenticate')
     @domains_configured
     @exception_translated('assertion')
-    def authenticate(self, request, user_id, password):
+    def authenticate(self, user_id, password):
         domain_id, driver, entity_id = (
             self._get_domain_driver_and_entity_id(user_id))
         ref = driver.authenticate(entity_id, password)
@@ -937,9 +929,8 @@ class Manager(manager.Manager):
         user = user_ref.copy()
         if 'password' in user:
             validators.validate_password(user['password'])
-        user['name'] = clean.user_name(user['name'])
+        user['name'] = user['name'].strip()
         user.setdefault('enabled', True)
-        user['enabled'] = clean.user_enabled(user['enabled'])
         domain_id = user['domain_id']
         PROVIDERS.resource_api.get_domain(domain_id)
 
@@ -1025,7 +1016,7 @@ class Manager(manager.Manager):
                 try:
                     filter_['comparator'] = operators[op]
                 except KeyError:
-                    raise exception.InvalidOperatorError(op)
+                    raise exception.InvalidOperatorError(_op=op)
         return hints
 
     def _handle_shadow_and_local_users(self, driver, hints):
@@ -1087,9 +1078,7 @@ class Manager(manager.Manager):
         if 'password' in user:
             validators.validate_password(user['password'])
         if 'name' in user:
-            user['name'] = clean.user_name(user['name'])
-        if 'enabled' in user:
-            user['enabled'] = clean.user_enabled(user['enabled'])
+            user['name'] = user['name'].strip()
         if 'id' in user:
             if user_id != user['id']:
                 raise exception.ValidationError(_('Cannot change user ID'))
@@ -1174,7 +1163,7 @@ class Manager(manager.Manager):
         # the underlying driver so that it could conform to rules set down by
         # that particular driver type.
         group['id'] = uuid.uuid4().hex
-        group['name'] = clean.group_name(group['name'])
+        group['name'] = group['name'].strip()
         ref = driver.create_group(group['id'], group)
 
         notifications.Audit.created(self._GROUP, group['id'], initiator)
@@ -1209,7 +1198,7 @@ class Manager(manager.Manager):
             self._get_domain_driver_and_entity_id(group_id))
         group = self._clear_domain_id_if_domain_unaware(driver, group)
         if 'name' in group:
-            group['name'] = clean.group_name(group['name'])
+            group['name'] = group['name'].strip()
         ref = driver.update_group(entity_id, group)
         self.get_group.invalidate(self, group_id)
         notifications.Audit.updated(self._GROUP, group_id, initiator)
@@ -1377,12 +1366,12 @@ class Manager(manager.Manager):
                                                 group_entity_id)
 
     @domains_configured
-    def change_password(self, request, user_id, original_password,
+    def change_password(self, user_id, original_password,
                         new_password, initiator=None):
 
         # authenticate() will raise an AssertionError if authentication fails
         try:
-            self.authenticate(request, user_id, original_password)
+            self.authenticate(user_id, original_password)
         except exception.PasswordExpired:
             # If a password has expired, we want users to be able to change it
             pass
