@@ -12,14 +12,17 @@
 
 from keystone.common import validation
 from keystone.common.validation import parameter_types
+import keystone.conf
 from keystone.identity.backends import resource_options as ro
 
 
-# NOTE(lhcheng): the max length is not applicable since it is specific
-# to the SQL backend, LDAP does not have length limitation.
+CONF = keystone.conf.CONF
+
+
 _identity_name = {
     'type': 'string',
     'minLength': 1,
+    'maxLength': 255,
     'pattern': '[\S]+'
 }
 
@@ -78,3 +81,27 @@ group_update = {
     'minProperties': 1,
     'additionalProperties': True
 }
+
+_password_change_properties = {
+    'original_password': {
+        'type': 'string'
+    },
+    'password': {
+        'type': 'string'
+    }
+}
+if getattr(CONF, 'strict_password_check', None):
+    _password_change_properties['password']['maxLength'] = \
+        CONF.identity.max_password_length
+
+if getattr(CONF, 'security_compliance', None):
+    if getattr(CONF.security_compliance, 'password_regex', None):
+        _password_change_properties['password']['pattern'] = \
+            CONF.security_compliance.password_regex
+
+password_change = {
+    'type': 'object',
+    'properties': _password_change_properties,
+    'required': ['original_password', 'password'],
+    'additionalProperties': False
+}
