@@ -143,7 +143,7 @@ def _assert_rbac_enforcement_called(resp):
     # assert is intended to be used to ensure code during development works
     # as expected, it is fine to be optimized out with `python -O`
     msg = ('PROGRAMMING ERROR: enforcement (`keystone.common.rbac_enforcer.'
-           'enforcer.RBACKEnforcer.enforce_call()`) has not been called; API '
+           'enforcer.RBACEnforcer.enforce_call()`) has not been called; API '
            'is unenforced.')
     g = flask.g
     # NOTE(morgan): OPTIONS is a special case and is handled by flask
@@ -604,6 +604,12 @@ class ResourceBase(flask_restful.Resource):
         ref['id'] = uuid.uuid4().hex
         return ref
 
+    @staticmethod
+    def _validate_id_format(id):
+        uval = uuid.UUID(id).hex
+        if uval != id:
+            raise ValueError('badly formed hexadecimal UUID value')
+
     @classmethod
     def _require_matching_id(cls, ref):
         """Ensure the value matches the reference's ID, if any."""
