@@ -13,13 +13,13 @@
 # under the License.
 
 import datetime
-import functools
 import uuid
 
-import freezegun
+import fixtures
 import mock
 from oslo_db import exception as db_exception
 from oslo_db import options
+from oslo_log import log
 from six.moves import range
 import sqlalchemy
 from sqlalchemy import exc
@@ -28,6 +28,7 @@ from testtools import matchers
 from keystone.common import driver_hints
 from keystone.common import provider_api
 from keystone.common import sql
+from keystone.common.sql import core
 import keystone.conf
 from keystone.credential.providers import fernet as credential_provider
 from keystone import exception
@@ -43,9 +44,7 @@ from keystone.tests.unit.ksfixtures import database
 from keystone.tests.unit.limit import test_backends as limit_tests
 from keystone.tests.unit.policy import test_backends as policy_tests
 from keystone.tests.unit.resource import test_backends as resource_tests
-from keystone.tests.unit.token import test_backends as token_tests
 from keystone.tests.unit.trust import test_backends as trust_tests
-from keystone.token.persistence.backends import sql as token_sql
 from keystone.trust.backends import sql as trust_sql
 
 
@@ -71,14 +70,126 @@ class SqlTests(unit.SQLDriverOverrides, unit.TestCase):
         return config_files
 
 
+class DataTypeRoundTrips(SqlTests):
+    def test_json_blob_roundtrip(self):
+        """Test round-trip of a JSON data structure with JsonBlob."""
+        with sql.session_for_read() as session:
+            val = session.scalar(
+                sqlalchemy.select(
+                    [sqlalchemy.literal({"key": "value"}, type_=core.JsonBlob)]
+                )
+            )
+
+        self.assertEqual({"key": "value"}, val)
+
+    def test_json_blob_sql_null(self):
+        """Test that JsonBlob can accommodate a SQL NULL value in a result set.
+
+        SQL NULL may be handled by JsonBlob in the case where a table is
+        storing NULL in a JsonBlob column, as several models use this type
+        in a column that is nullable.   It also comes back when the column
+        is left NULL from being in an OUTER JOIN.  In Python, this means
+        the None constant is handled by the datatype.
+
+        """
+        with sql.session_for_read() as session:
+            val = session.scalar(
+                sqlalchemy.select(
+                    [sqlalchemy.cast(sqlalchemy.null(), type_=core.JsonBlob)]
+                )
+            )
+
+        self.assertIsNone(val)
+
+    def test_json_blob_python_none(self):
+        """Test that JsonBlob round-trips a Python None.
+
+        This is where JSON datatypes get a little nutty, in that JSON has
+        a 'null' keyword, and JsonBlob right now will persist Python None
+        as the json string 'null', not SQL NULL.
+
+        """
+        with sql.session_for_read() as session:
+            val = session.scalar(
+                sqlalchemy.select(
+                    [sqlalchemy.literal(None, type_=core.JsonBlob)]
+                )
+            )
+
+        self.assertIsNone(val)
+
+    def test_json_blob_python_none_renders(self):
+        """Test that JsonBlob actually renders JSON 'null' for Python None."""
+        with sql.session_for_read() as session:
+            val = session.scalar(
+                sqlalchemy.select(
+                    [
+                        sqlalchemy.cast(
+                            sqlalchemy.literal(None, type_=core.JsonBlob),
+                            sqlalchemy.String,
+                        )
+                    ]
+                )
+            )
+
+        self.assertEqual("null", val)
+
+    def test_datetimeint_roundtrip(self):
+        """Test round-trip of a Python datetime with DateTimeInt."""
+        with sql.session_for_read() as session:
+            datetime_value = datetime.datetime(2019, 5, 15, 10, 17, 55)
+            val = session.scalar(
+                sqlalchemy.select(
+                    [
+                        sqlalchemy.literal(
+                            datetime_value, type_=core.DateTimeInt
+                        ),
+                    ]
+                )
+            )
+
+        self.assertEqual(datetime_value, val)
+
+    def test_datetimeint_persistence(self):
+        """Test integer persistence with DateTimeInt."""
+        with sql.session_for_read() as session:
+            datetime_value = datetime.datetime(2019, 5, 15, 10, 17, 55)
+            val = session.scalar(
+                sqlalchemy.select(
+                    [
+                        sqlalchemy.cast(
+                            sqlalchemy.literal(
+                                datetime_value, type_=core.DateTimeInt
+                            ),
+                            sqlalchemy.Integer
+                        )
+                    ]
+                )
+            )
+
+        self.assertEqual(1557915475000000, val)
+
+    def test_datetimeint_python_none(self):
+        """Test round-trip of a Python None with DateTimeInt."""
+        with sql.session_for_read() as session:
+            val = session.scalar(
+                sqlalchemy.select(
+                    [
+                        sqlalchemy.literal(None, type_=core.DateTimeInt),
+                    ]
+                )
+            )
+
+        self.assertIsNone(val)
+
+
 class SqlModels(SqlTests):
 
-    def select_table(self, name):
+    def load_table(self, name):
         table = sqlalchemy.Table(name,
                                  sql.ModelBase.metadata,
                                  autoload=True)
-        s = sqlalchemy.select([table])
-        return s
+        return table
 
     def assertExpectedSchema(self, table, expected_schema):
         """Assert that a table's schema is what we expect.
@@ -112,14 +223,14 @@ class SqlModels(SqlTests):
             self.assertExpectedSchema('table_name', cols)
 
         """
-        table = self.select_table(table)
+        table = self.load_table(table)
 
         actual_schema = []
         for column in table.c:
             if isinstance(column.type, sql.Boolean):
                 default = None
-                if column._proxies[0].default:
-                    default = column._proxies[0].default.arg
+                if column.default:
+                    default = column.default.arg
                 actual_schema.append((column.name, type(column.type), default))
             elif (hasattr(column.type, 'length') and
                     not isinstance(column.type, sql.Enum)):
@@ -157,7 +268,6 @@ class SqlModels(SqlTests):
     def test_password_model(self):
         cols = (('id', sql.Integer, None),
                 ('local_user_id', sql.Integer, None),
-                ('password', sql.String, 128),
                 ('password_hash', sql.String, 255),
                 ('created_at', sql.DateTime, None),
                 ('expires_at', sql.DateTime, None),
@@ -280,24 +390,13 @@ class SqlIdentity(SqlTests,
         role_member = unit.new_role_ref()
         PROVIDERS.role_api.create_role(role_member['id'], role_member)
         PROVIDERS.assignment_api.add_role_to_user_and_project(
-            user['id'], self.tenant_bar['id'], role_member['id']
+            user['id'], self.project_bar['id'], role_member['id']
         )
         PROVIDERS.identity_api.delete_user(user['id'])
         self.assertRaises(exception.UserNotFound,
                           PROVIDERS.assignment_api.list_projects_for_user,
                           user['id'])
 
-    def test_create_null_user_name(self):
-        user = unit.new_user_ref(name=None,
-                                 domain_id=CONF.identity.default_domain_id)
-        self.assertRaises(exception.ValidationError,
-                          PROVIDERS.identity_api.create_user,
-                          user)
-        self.assertRaises(exception.UserNotFound,
-                          PROVIDERS.identity_api.get_user_by_name,
-                          user['name'],
-                          CONF.identity.default_domain_id)
-
     def test_create_user_case_sensitivity(self):
         # user name case sensitivity is down to the fact that it is marked as
         # an SQL UNIQUE column, which may not be valid for other backends, like
@@ -332,11 +431,11 @@ class SqlIdentity(SqlTests,
         role_member = unit.new_role_ref()
         PROVIDERS.role_api.create_role(role_member['id'], role_member)
         PROVIDERS.assignment_api.add_role_to_user_and_project(
-            user['id'], self.tenant_bar['id'], role_member['id']
+            user['id'], self.project_bar['id'], role_member['id']
         )
-        PROVIDERS.resource_api.delete_project(self.tenant_bar['id'])
-        tenants = PROVIDERS.assignment_api.list_projects_for_user(user['id'])
-        self.assertEqual([], tenants)
+        PROVIDERS.resource_api.delete_project(self.project_bar['id'])
+        projects = PROVIDERS.assignment_api.list_projects_for_user(user['id'])
+        self.assertEqual([], projects)
 
     def test_update_project_returns_extra(self):
         """Test for backward compatibility with an essex/folsom bug.
@@ -355,7 +454,7 @@ class SqlIdentity(SqlTests,
         project[arbitrary_key] = arbitrary_value
         ref = PROVIDERS.resource_api.create_project(project['id'], project)
         self.assertEqual(arbitrary_value, ref[arbitrary_key])
-        self.assertIsNone(ref.get('extra'))
+        self.assertNotIn('extra', ref)
 
         ref['name'] = uuid.uuid4().hex
         ref = PROVIDERS.resource_api.update_project(ref['id'], ref)
@@ -379,14 +478,14 @@ class SqlIdentity(SqlTests,
         del user["id"]
         ref = PROVIDERS.identity_api.create_user(user)
         self.assertEqual(arbitrary_value, ref[arbitrary_key])
-        self.assertIsNone(ref.get('password'))
-        self.assertIsNone(ref.get('extra'))
+        self.assertNotIn('password', ref)
+        self.assertNotIn('extra', ref)
 
         user['name'] = uuid.uuid4().hex
         user['password'] = uuid.uuid4().hex
         ref = PROVIDERS.identity_api.update_user(ref['id'], user)
-        self.assertIsNone(ref.get('password'))
-        self.assertIsNone(ref['extra'].get('password'))
+        self.assertNotIn('password', ref)
+        self.assertNotIn('password', ref['extra'])
         self.assertEqual(arbitrary_value, ref[arbitrary_key])
         self.assertEqual(arbitrary_value, ref['extra'][arbitrary_key])
 
@@ -689,8 +788,10 @@ class SqlIdentity(SqlTests,
                               ref_id)
 
             # Deleting list of projects that includes a non-existing project
-            # should be silent
-            driver.delete_projects_from_ids([ref_id])
+            # should be silent. The root domain <<keystone.domain.root>> can't
+            # be deleted.
+            if ref_id != resource.NULL_DOMAIN_ID:
+                driver.delete_projects_from_ids([ref_id])
 
         _exercise_project_api(uuid.uuid4().hex)
         _exercise_project_api(resource.NULL_DOMAIN_ID)
@@ -731,6 +832,70 @@ class SqlIdentity(SqlTests,
         # users fetched.
         self.assertNotEqual(len(first_call_users), len(second_call_users))
         self.assertEqual(first_call_counter, counter.calls)
+        self.assertEqual(3, counter.calls)
+
+    def test_check_project_depth(self):
+        # Create a 3 level project tree:
+        #
+        # default_domain
+        #       |
+        #   project_1
+        #       |
+        #   project_2
+        project_1 = unit.new_project_ref(
+            domain_id=CONF.identity.default_domain_id)
+        PROVIDERS.resource_api.create_project(project_1['id'], project_1)
+        project_2 = unit.new_project_ref(
+            domain_id=CONF.identity.default_domain_id,
+            parent_id=project_1['id'])
+        PROVIDERS.resource_api.create_project(project_2['id'], project_2)
+
+        # if max_depth is None or >= current project depth, return nothing.
+        resp = PROVIDERS.resource_api.check_project_depth(max_depth=None)
+        self.assertIsNone(resp)
+        resp = PROVIDERS.resource_api.check_project_depth(max_depth=3)
+        self.assertIsNone(resp)
+        resp = PROVIDERS.resource_api.check_project_depth(max_depth=4)
+        self.assertIsNone(resp)
+        # if max_depth < current project depth, raise LimitTreeExceedError
+        self.assertRaises(exception.LimitTreeExceedError,
+                          PROVIDERS.resource_api.check_project_depth,
+                          2)
+
+    def test_update_user_with_stale_data_forces_retry(self):
+        # Capture log output so we know oslo.db attempted a retry
+        log_fixture = self.useFixture(fixtures.FakeLogger(level=log.DEBUG))
+
+        # Create a new user
+        user_dict = unit.new_user_ref(
+            domain_id=CONF.identity.default_domain_id)
+        new_user_dict = PROVIDERS.identity_api.create_user(user_dict)
+
+        side_effects = [
+            # Raise a StaleDataError simulating that another client has
+            # updated the user's password while this client's request was
+            # being processed
+            sqlalchemy.orm.exc.StaleDataError,
+            # The oslo.db library will retry the request, so the second
+            # time this method is called let's return a valid session
+            # object
+            sql.session_for_write()
+        ]
+        with mock.patch('keystone.common.sql.session_for_write') as m:
+            m.side_effect = side_effects
+
+            # Update a user's attribute, the first attempt will fail but
+            # oslo.db will handle the exception and retry, the second attempt
+            # will succeed
+            new_user_dict['email'] = uuid.uuid4().hex
+            PROVIDERS.identity_api.update_user(
+                new_user_dict['id'], new_user_dict)
+
+        # Make sure oslo.db retried the update by checking the log output
+        expected_log_message = (
+            'Performing DB retry for function keystone.identity.backends.sql'
+        )
+        self.assertIn(expected_log_message, log_fixture.output)
 
 
 class SqlTrust(SqlTests, trust_tests.TrustTests):
@@ -745,152 +910,26 @@ class SqlTrust(SqlTests, trust_tests.TrustTests):
             self.assertEqual(trust_ref.expires_at, trust_ref.expires_at_int)
 
 
-class SqlToken(SqlTests, token_tests.TokenTests):
-    def test_token_revocation_list_uses_right_columns(self):
-        # This query used to be heavy with too many columns. We want
-        # to make sure it is only running with the minimum columns
-        # necessary.
-
-        expected_query_args = (token_sql.TokenModel.id,
-                               token_sql.TokenModel.expires,
-                               token_sql.TokenModel.extra,)
-
-        with mock.patch.object(token_sql, 'sql') as mock_sql:
-            tok = token_sql.Token()
-            tok.list_revoked_tokens()
-
-        mock_query = mock_sql.session_for_read().__enter__().query
-        mock_query.assert_called_with(*expected_query_args)
-
-    def test_flush_expired_tokens_batch(self):
-        # TODO(dstanek): This test should be rewritten to be less
-        # brittle. The code will likely need to be changed first. I
-        # just copied the spirit of the existing test when I rewrote
-        # mox -> mock. These tests are brittle because they have the
-        # call structure for SQLAlchemy encoded in them.
-
-        # test sqlite dialect
-        with mock.patch.object(token_sql, 'sql') as mock_sql:
-            mock_sql.get_session().bind.dialect.name = 'sqlite'
-            tok = token_sql.Token()
-            tok.flush_expired_tokens()
-
-        filter_mock = mock_sql.get_session().query().filter()
-        self.assertFalse(filter_mock.limit.called)
-        self.assertTrue(filter_mock.delete.called_once)
-
-    def test_flush_expired_tokens_batch_mysql(self):
-        # test mysql dialect, we don't need to test IBM DB SA separately, since
-        # other tests below test the differences between how they use the batch
-        # strategy
-        with mock.patch.object(token_sql, 'sql') as mock_sql:
-            mock_sql.session_for_write().__enter__(
-            ).query().filter().delete.return_value = 0
-
-            mock_sql.session_for_write().__enter__(
-            ).bind.dialect.name = 'mysql'
-
-            tok = token_sql.Token()
-            expiry_mock = mock.Mock()
-            ITERS = [1, 2, 3]
-            expiry_mock.return_value = iter(ITERS)
-            token_sql._expiry_range_batched = expiry_mock
-            tok.flush_expired_tokens()
-
-            # The expiry strategy is only invoked once, the other calls are via
-            # the yield return.
-            self.assertEqual(1, expiry_mock.call_count)
-
-            mock_delete = mock_sql.session_for_write().__enter__(
-            ).query().filter().delete
-
-            self.assertThat(mock_delete.call_args_list,
-                            matchers.HasLength(len(ITERS)))
-
-    def test_expiry_range_batched(self):
-        upper_bound_mock = mock.Mock(side_effect=[1, "final value"])
-        sess_mock = mock.Mock()
-        query_mock = sess_mock.query().filter().order_by().offset().limit()
-        query_mock.one.side_effect = [['test'], sql.NotFound()]
-        for i, x in enumerate(token_sql._expiry_range_batched(sess_mock,
-                                                              upper_bound_mock,
-                                                              batch_size=50)):
-            if i == 0:
-                # The first time the batch iterator returns, it should return
-                # the first result that comes back from the database.
-                self.assertEqual('test', x)
-            elif i == 1:
-                # The second time, the database range function should return
-                # nothing, so the batch iterator returns the result of the
-                # upper_bound function
-                self.assertEqual("final value", x)
-            else:
-                self.fail("range batch function returned more than twice")
-
-    def test_expiry_range_strategy_sqlite(self):
-        tok = token_sql.Token()
-        sqlite_strategy = tok._expiry_range_strategy('sqlite')
-        self.assertEqual(token_sql._expiry_range_all, sqlite_strategy)
-
-    def test_expiry_range_strategy_ibm_db_sa(self):
-        tok = token_sql.Token()
-        db2_strategy = tok._expiry_range_strategy('ibm_db_sa')
-        self.assertIsInstance(db2_strategy, functools.partial)
-        self.assertEqual(token_sql._expiry_range_batched, db2_strategy.func)
-        self.assertEqual({'batch_size': 100}, db2_strategy.keywords)
-
-    def test_expiry_range_strategy_mysql(self):
-        tok = token_sql.Token()
-        mysql_strategy = tok._expiry_range_strategy('mysql')
-        self.assertIsInstance(mysql_strategy, functools.partial)
-        self.assertEqual(token_sql._expiry_range_batched, mysql_strategy.func)
-        self.assertEqual({'batch_size': 1000}, mysql_strategy.keywords)
-
-    def test_expiry_range_with_allow_expired(self):
-        window_secs = 200
-        self.config_fixture.config(group='token',
-                                   allow_expired_window=window_secs)
-
-        tok = token_sql.Token()
-        time = datetime.datetime.utcnow()
-
-        with freezegun.freeze_time(time):
-            # unknown strategy just ensures we are getting the dumbest strategy
-            # that will remove everything in one go
-            strategy = tok._expiry_range_strategy('unkown')
-            upper_bound_func = token_sql._expiry_upper_bound_func
-
-            # session is ignored for dumb strategy
-            expiry_times = list(strategy(session=None,
-                                         upper_bound_func=upper_bound_func))
-
-            # basically just ensure that we are removing things in the past
-            delta = datetime.timedelta(seconds=window_secs)
-            previous_time = datetime.datetime.utcnow() - delta
-
-        self.assertEqual([previous_time], expiry_times)
-
-
 class SqlCatalog(SqlTests, catalog_tests.CatalogTests):
 
     _legacy_endpoint_id_in_endpoint = True
     _enabled_default_to_true_when_creating_endpoint = True
 
-    def test_catalog_ignored_malformed_urls(self):
+    def test_get_v3_catalog_project_non_exist(self):
         service = unit.new_service_ref()
         PROVIDERS.catalog_api.create_service(service['id'], service)
 
-        malformed_url = "http://192.168.1.104:8774/v2/$(tenant)s"
+        malformed_url = "http://192.168.1.104:8774/v2/$(project)s"
         endpoint = unit.new_endpoint_ref(service_id=service['id'],
                                          url=malformed_url,
                                          region_id=None)
         PROVIDERS.catalog_api.create_endpoint(endpoint['id'], endpoint.copy())
+        self.assertRaises(exception.ProjectNotFound,
+                          PROVIDERS.catalog_api.get_v3_catalog,
+                          'fake-user',
+                          'fake-project')
 
-        # NOTE(dstanek): there are no valid URLs, so nothing is in the catalog
-        catalog = PROVIDERS.catalog_api.get_catalog('fake-user', 'fake-tenant')
-        self.assertEqual({}, catalog)
-
-    def test_get_catalog_with_empty_public_url(self):
+    def test_get_v3_catalog_with_empty_public_url(self):
         service = unit.new_service_ref()
         PROVIDERS.catalog_api.create_service(service['id'], service)
 
@@ -898,13 +937,12 @@ class SqlCatalog(SqlTests, catalog_tests.CatalogTests):
                                          region_id=None)
         PROVIDERS.catalog_api.create_endpoint(endpoint['id'], endpoint.copy())
 
-        catalog = PROVIDERS.catalog_api.get_catalog('user', 'tenant')
-        catalog_endpoint = catalog[endpoint['region_id']][service['type']]
+        catalog = PROVIDERS.catalog_api.get_v3_catalog(self.user_foo['id'],
+                                                       self.project_bar['id'])
+        catalog_endpoint = catalog[0]
         self.assertEqual(service['name'], catalog_endpoint['name'])
-        self.assertEqual(endpoint['id'], catalog_endpoint['id'])
-        self.assertEqual('', catalog_endpoint['publicURL'])
-        self.assertIsNone(catalog_endpoint.get('adminURL'))
-        self.assertIsNone(catalog_endpoint.get('internalURL'))
+        self.assertEqual(service['id'], catalog_endpoint['id'])
+        self.assertEqual([], catalog_endpoint['endpoints'])
 
     def test_create_endpoint_region_returns_not_found(self):
         service = unit.new_service_ref()
@@ -965,7 +1003,7 @@ class SqlCatalog(SqlTests, catalog_tests.CatalogTests):
                           region['id'])
 
     def test_v3_catalog_domain_scoped_token(self):
-        # test the case that tenant_id is None.
+        # test the case that project_id is None.
         srv_1 = unit.new_service_ref()
         PROVIDERS.catalog_api.create_service(srv_1['id'], srv_1)
         endpoint_1 = unit.new_endpoint_ref(service_id=srv_1['id'],
@@ -1003,10 +1041,10 @@ class SqlCatalog(SqlTests, catalog_tests.CatalogTests):
         # create endpoint-project association.
         PROVIDERS.catalog_api.add_endpoint_to_project(
             endpoint_1['id'],
-            self.tenant_bar['id'])
+            self.project_bar['id'])
 
         catalog_ref = PROVIDERS.catalog_api.get_v3_catalog(
-            uuid.uuid4().hex, self.tenant_bar['id']
+            uuid.uuid4().hex, self.project_bar['id']
         )
         self.assertThat(catalog_ref, matchers.HasLength(1))
         self.assertThat(catalog_ref[0]['endpoints'], matchers.HasLength(1))
@@ -1028,7 +1066,7 @@ class SqlCatalog(SqlTests, catalog_tests.CatalogTests):
         PROVIDERS.catalog_api.create_service(srv_2['id'], srv_2)
 
         catalog_ref = PROVIDERS.catalog_api.get_v3_catalog(
-            uuid.uuid4().hex, self.tenant_bar['id']
+            uuid.uuid4().hex, self.project_bar['id']
         )
         self.assertThat(catalog_ref, matchers.HasLength(2))
         srv_id_list = [catalog_ref[0]['id'], catalog_ref[1]['id']]
@@ -1047,24 +1085,6 @@ class SqlImpliedRoles(SqlTests, assignment_tests.ImpliedRoleTests):
     pass
 
 
-class SqlTokenCacheInvalidationWithUUID(SqlTests,
-                                        token_tests.TokenCacheInvalidation):
-    def setUp(self):
-        super(SqlTokenCacheInvalidationWithUUID, self).setUp()
-        self._create_test_data()
-
-    def config_overrides(self):
-        super(SqlTokenCacheInvalidationWithUUID, self).config_overrides()
-        # NOTE(lbragstad): The TokenCacheInvalidation tests are coded to work
-        # against a persistent token backend. Only run these with token
-        # providers that issue persistent tokens.
-        self.config_fixture.config(group='token', provider='uuid')
-
-
-# NOTE(lbragstad): The Fernet token provider doesn't persist tokens in a
-# backend, so running the TokenCacheInvalidation tests here doesn't make sense.
-
-
 class SqlFilterTests(SqlTests, identity_tests.FilterTests):
 
     def clean_up_entities(self):
@@ -1342,5 +1362,9 @@ class SqlLimit(SqlTests, limit_tests.LimitTests):
             service_id=self.service_one['id'],
             region_id=self.region_two['id'],
             resource_name='snapshot', default_limit=10, id=uuid.uuid4().hex)
+        registered_limit_3 = unit.new_registered_limit_ref(
+            service_id=self.service_one['id'],
+            region_id=self.region_two['id'],
+            resource_name='backup', default_limit=10, id=uuid.uuid4().hex)
         PROVIDERS.unified_limit_api.create_registered_limits(
-            [registered_limit_1, registered_limit_2])
+            [registered_limit_1, registered_limit_2, registered_limit_3])
