@@ -88,7 +88,8 @@ class TestTemplatedCatalog(unit.TestCase, catalog_tests.CatalogTests):
             "Templated backend doesn't have disabled endpoints")
 
     def assert_catalogs_equal(self, expected, observed):
-        sort_key = lambda d: d['id']
+        def sort_key(d):
+            return d['id']
         for e, o in zip(sorted(expected, key=sort_key),
                         sorted(observed, key=sort_key)):
             expected_endpoints = e.pop('endpoints')
@@ -129,6 +130,64 @@ class TestTemplatedCatalog(unit.TestCase, catalog_tests.CatalogTests):
              'id': '1'}]
         self.assert_catalogs_equal(exp_catalog, catalog_ref)
 
+    def test_get_multi_region_v3_catalog(self):
+        user_id = uuid.uuid4().hex
+        project_id = uuid.uuid4().hex
+
+        catalog_api = PROVIDERS.catalog_api
+
+        # Load the multi-region catalog.
+        catalog_api._load_templates(
+            unit.dirs.tests('default_catalog_multi_region.templates'))
+
+        catalog_ref = catalog_api.get_v3_catalog(user_id, project_id)
+        exp_catalog = [
+            {'endpoints': [
+                {'interface': 'admin',
+                 'region': 'RegionOne',
+                 'url': 'http://region-one:8774/v1.1/%s' % project_id},
+                {'interface': 'public',
+                 'region': 'RegionOne',
+                 'url': 'http://region-one:8774/v1.1/%s' % project_id},
+                {'interface': 'internal',
+                 'region': 'RegionOne',
+                 'url': 'http://region-one:8774/v1.1/%s' % project_id},
+                {'interface': 'admin',
+                 'region': 'RegionTwo',
+                 'url': 'http://region-two:8774/v1.1/%s' % project_id},
+                {'interface': 'public',
+                 'region': 'RegionTwo',
+                 'url': 'http://region-two:8774/v1.1/%s' % project_id},
+                {'interface': 'internal',
+                 'region': 'RegionTwo',
+                 'url': 'http://region-two:8774/v1.1/%s' % project_id}],
+                'type': 'compute',
+                'name': "'Compute Service'",
+                'id': '2'},
+            {'endpoints': [
+                {'interface': 'admin',
+                 'region': 'RegionOne',
+                 'url': 'http://region-one:35357/v3'},
+                {'interface': 'public',
+                 'region': 'RegionOne',
+                 'url': 'http://region-one:5000/v3'},
+                {'interface': 'internal',
+                 'region': 'RegionOne',
+                 'url': 'http://region-one:35357/v3'},
+                {'interface': 'admin',
+                 'region': 'RegionTwo',
+                 'url': 'http://region-two:35357/v3'},
+                {'interface': 'public',
+                 'region': 'RegionTwo',
+                 'url': 'http://region-two:5000/v3'},
+                {'interface': 'internal',
+                 'region': 'RegionTwo',
+                 'url': 'http://region-two:35357/v3'}],
+                'type': 'identity',
+                'name': "'Identity Service'",
+                'id': '1'}]
+        self.assert_catalogs_equal(exp_catalog, catalog_ref)
+
     def test_get_catalog_ignores_endpoints_with_invalid_urls(self):
         user_id = uuid.uuid4().hex
         project_id = None
@@ -191,6 +250,9 @@ class TestTemplatedCatalog(unit.TestCase, catalog_tests.CatalogTests):
     def test_invalidate_cache_when_updating_region(self):
         self.skip_test_overrides(BROKEN_WRITE_FUNCTIONALITY_MSG)
 
+    def test_update_region_extras(self):
+        self.skip_test_overrides(BROKEN_WRITE_FUNCTIONALITY_MSG)
+
     def test_create_region_with_duplicate_id(self):
         self.skip_test_overrides(BROKEN_WRITE_FUNCTIONALITY_MSG)
 
