@@ -93,7 +93,7 @@ class TestValidate(unit.TestCase):
             user_ref['password_expires_at'], token.user['password_expires_at']
         )
 
-    def test_validate_v3_token_federated_info(self):
+    def _test_validate_v3_token_federted_info(self, group_ids):
         # Check the user fields in the token result when use validate_v3_token
         # when the token has federated info.
 
@@ -107,7 +107,6 @@ class TestValidate(unit.TestCase):
 
         method_names = ['mapped']
 
-        group_ids = [uuid.uuid4().hex, ]
         idp_id = uuid.uuid4().hex
         idp_ref = {
             'id': idp_id,
@@ -137,6 +136,18 @@ class TestValidate(unit.TestCase):
         self.assertEqual(idp_id, token.identity_provider_id)
         self.assertEqual(protocol, token.protocol_id)
 
+    def test_validate_v3_token_federated_info(self):
+        # Check the user fields in the token result when use validate_v3_token
+        # when the token has federated info.
+
+        group_ids = [uuid.uuid4().hex, ]
+        self._test_validate_v3_token_federted_info(group_ids)
+
+    def test_validate_v3_token_federated_info_empty_group(self):
+        # check when federated users got empty group ids
+
+        self._test_validate_v3_token_federted_info([])
+
     def test_validate_v3_token_trust(self):
         # Check the trust fields in the token result when use validate_v3_token
         # when the token has trust info.
@@ -201,6 +212,14 @@ class TestValidate(unit.TestCase):
         )
 
 
+class TestValidateWithoutCache(TestValidate):
+
+    def config_overrides(self):
+        super(TestValidateWithoutCache, self).config_overrides()
+        self.config_fixture.config(group='token', caching=False)
+        self.config_fixture.config(group='token', cache_on_issue=False)
+
+
 class TestTokenFormatter(unit.TestCase):
     def test_restore_padding(self):
         # 'a' will result in '==' padding, 'aa' will result in '=' padding, and
