@@ -14,6 +14,8 @@
 
 from oslo_utils import timeutils
 from six.moves import range
+
+import sqlalchemy
 from sqlalchemy.ext.hybrid import hybrid_property
 
 from keystone.common import sql
@@ -30,7 +32,8 @@ class TrustModel(sql.ModelBase, sql.ModelDictMixinWithExtras):
     __tablename__ = 'trust'
     attributes = ['id', 'trustor_user_id', 'trustee_user_id',
                   'project_id', 'impersonation', 'expires_at',
-                  'remaining_uses', 'deleted_at']
+                  'remaining_uses', 'deleted_at', 'redelegated_trust_id',
+                  'redelegation_count']
     id = sql.Column(sql.String(64), primary_key=True)
     # user id of owner
     trustor_user_id = sql.Column(sql.String(64), nullable=False,)
@@ -42,6 +45,8 @@ class TrustModel(sql.ModelBase, sql.ModelDictMixinWithExtras):
     _expires_at = sql.Column('expires_at', sql.DateTime)
     expires_at_int = sql.Column(sql.DateTimeInt(), nullable=True)
     remaining_uses = sql.Column(sql.Integer, nullable=True)
+    redelegated_trust_id = sql.Column(sql.String(64), nullable=True)
+    redelegation_count = sql.Column(sql.Integer, nullable=True)
     extra = sql.Column(sql.JsonBlob())
     __table_args__ = (sql.UniqueConstraint(
                       'trustor_user_id', 'trustee_user_id', 'project_id',
@@ -188,3 +193,23 @@ class Trust(base.TrustDriverBase):
             trusts = query.filter_by(project_id=project_id)
             for trust_ref in trusts:
                 trust_ref.deleted_at = timeutils.utcnow()
+
+    def flush_expired_and_soft_deleted_trusts(self, project_id=None,
+                                              trustor_user_id=None,
+                                              trustee_user_id=None,
+                                              date=None):
+        with sql.session_for_write() as session:
+            query = session.query(TrustModel)
+            query = query.\
+                filter(sqlalchemy.or_(TrustModel.deleted_at.isnot(None),
+                                      sqlalchemy.and_(
+                                          TrustModel.expires_at.isnot(None),
+                                          TrustModel.expires_at < date)))
+            if project_id:
+                query = query.filter_by(project_id=project_id)
+            if trustor_user_id:
+                query = query.filter_by(trustor_user_id=trustor_user_id)
+            if trustee_user_id:
+                query = query.filter_by(trustee_user_id=trustee_user_id)
+            query.delete(synchronize_session=False)
+            session.flush()
